

function [output_matrix] = backward_solve_3(input_matrix)   	%takes input matrice and gives solution matrice
	
	action_matrix=	[	1 1 0 1 0 0 0 0 0;
  						1 1 1 0 1 0 0 0 0;
						0 1 1 0 0 1 0 0 0;
						1 0 0 1 1 0 1 0 0;
						0 1 0 1 1 1 0 1 0;
						0 0 1 0 1 1 0 0 1;
						0 0 0 1 0 0 1 1 0;
						0 0 0 0 1 0 1 1 1;
						0 0 0 0 0 1 0 1 1;];	  	
	%where action_matrix(i,j) = 1 ⇐⇒ button i is toggled on pressing button j
	csize=3;						  	%columns in initial given matrix
	rsize=3;								%rows in initial given matrix
	aug_matrix=transpose(input_matrix);						%aug_matrix has column vector of input_matrix
	aug_matrix=vec(aug_matrix);
	[action_matrix aug_matrix]=rref(action_matrix,aug_matrix);    %rref in mod2
	output_matrix=reshape(aug_matrix,csize,rsize);				  %output_matrix contains solution to input_matrix
	output_matrix=transpose(output_matrix);

endfunction


function [action_matrix aug_matrix] = rowadd(row1,row2,action_matrix,aug_matrix)  %adding row1 to row2
colsize=columns(action_matrix); 
for k=1:colsize
	action_matrix(row2,k)=eleoperation(action_matrix(row1,k),action_matrix(row2,k));
endfor
aug_matrix(row2,1)=eleoperation(aug_matrix(row1,1),aug_matrix(row2,1));
endfunction




function f_value = eleoperation(i,j)											%returns mod2 operation on variables i and j
if(i==0 && j==0)
	f_value=0;
else
	f_value=!(i && j);
endif
endfunction



function [action_matrix aug_matrix] =rowinterchange(row1,row2,action_matrix,aug_matrix)   %interchanges row1 with row2
x=action_matrix(row1,:);
action_matrix(row1,:)=action_matrix(row2,:);
action_matrix(row2,:)=x;
y=aug_matrix(row1,1);
aug_matrix(row1,1)=aug_matrix(row2,1);
aug_matrix(row2,1)=y;
endfunction




function [action_matrix aug_matrix] =ref(action_matrix,aug_matrix)							%calculating reduced echlon form in mod2
colsize=columns(action_matrix); 
rowsize=rows(action_matrix);
pivot=0;
temp=-1;
found=0;
for i=1:colsize
	for j=(pivot+1):rowsize
		if(action_matrix(j,i)==1)
			if(temp==-1)
			    pivot=j;
			    temp=pivot-1;
			else
				temp=pivot;
				pivot=j;
			endif
		[action_matrix aug_matrix]=rowinterchange((temp+1),pivot,action_matrix,aug_matrix);
		pivot=temp+1;
		found=1;
		break;
		endif
	endfor
	if(found==0)
		continue;
	else
		for k=(pivot+1):rowsize
			if(action_matrix(k,i)==1)
				[action_matrix aug_matrix]=rowadd(pivot,k,action_matrix,aug_matrix);
			endif
		endfor
		found=0;
	endif
endfor
endfunction



function [action_matrix aug_matrix] =rref(action_matrix,aug_matrix)								%calculating row reduced echlon form
colsize=columns(action_matrix); 
rowsize=rows(action_matrix);
[action_matrix aug_matrix]=ref(action_matrix,aug_matrix);
pivot=0;
temp=-1;
found=0;
for i=colsize:-1:1
	for j=rowsize:-1:1
		if(action_matrix(j,i)==1)
			pivot=j;
		    found=1;
		break;
		endif
	endfor
	if(found==0)
		continue;
	else
		for k=(pivot-1):-1:1
			if(action_matrix(k,i)==1)
				[action_matrix aug_matrix]=rowadd(pivot,k,action_matrix,aug_matrix);
			endif
		endfor
		found=0;
	endif
endfor
endfunction





